@@ -1,12 +1,24 @@
-__metaclass__ = type
+# coding: utf-8
+from __future__ import unicode_literals
+from django.utils import six
 
-import time
-import base64
+if six.PY2:
+    def b(x):
+        return x
+else:
+    import codecs
+
+    def b(x):
+        return codecs.latin_1_encode(x)[0]
+__metaclass__ = type
 
 from django.conf import settings
 from django.contrib.auth.models import User
 from openid.consumer.consumer import SUCCESS
 from openid.extensions import ax, sreg
+from django.db import IntegrityError
+from django.utils.crypto import get_random_string
+
 
 from .constants import AX_MAPPINGS, AX_IS_VERIFIED
 from .models import UserOpenID
@@ -58,14 +70,27 @@ class OpenIDBackend(object):
 
     def create_user_from_openid(self, openid_response):
         email = self._get_email(openid_response)
-        username = base64.b64encode(str(time.time())).replace('=', '')
-        user = User.objects.create_user(username, email)
+        user = self.create_user_from_email(email)
         user.first_name = self._get_first_name(openid_response)
         user.last_name = self._get_last_name(openid_response)
         user.save()
         self.associate_openid(user, openid_response)
         return user
 
+    def create_user_from_email(self, email):
+        try:
+            username = email.split('@')[0]
+        except (AttributeError, IndexError):
+            username = get_random_string()
+        try:
+            user = User.objects.create_user(username, email)
+        except:
+            username = "{username}.{random_str}".format(
+                username=username, random_str=get_random_string(7))
+            user = User.objects.create_user(username, email)
+
+        return user
+
     def associate_openid(self, user, openid_response):
         user_openid = UserOpenID.by_identity_url(openid_response.identity_url)
         if not user_openid:
@@ -105,17 +130,17 @@ class OpenIDBackend(object):
 
     def _get_first_name(self, openid_response):
         fullname = self._get_full_name(openid_response)
-        if ' ' in fullname:
+        if fullname and ' ' in fullname:
             first_name, _, _ = fullname.rpartition(' ')
             return first_name
         return ''
 
     def _get_last_name(self, openid_response):
         fullname = self._get_full_name(openid_response)
-        if ' ' in fullname:
+        if fullname and ' ' in fullname:
             _, _, last_name = fullname.rpartition(' ')
             return last_name
-        return fullname
+        return ''
 
     def _get_ax(self, openid_response):
         return ax.FetchResponse.fromSuccessResponse(openid_response)
@@ -123,6 +148,6 @@ class OpenIDBackend(object):
     def _get_is_verified(self, openid_response):
         ax_response = self._get_ax(openid_response)
         ax_is_verified = ax_response.get(AX_MAPPINGS[AX_IS_VERIFIED][0])[0]
-        if ax_is_verified == u'True':
+        if ax_is_verified == 'True':
             return True
-        return False
\ No newline at end of file
+        return False
